<?php
class xoogu_simpleSocialSharing_admin extends xoogu_simpleSocialSharing{
	
	public function __construct(){
		$this->_options = get_option('xoogu-simple-social-sharing');
		$mainPluginFile=dirname(__FILE__).'/xoogu-simple-social-sharing.php';
		register_activation_hook($mainPluginFile, array($this, 'activate'));
		//register_deactivation_hook($mainPluginFile, array(&$this, 'deactivate'));
		register_uninstall_hook($mainPluginFile, array('xoogu_simpleSocialSharing_admin', 'uninstall')); //must be called statically
		add_action( 'wpmu_new_blog', array($this, 'new_blog'), 10, 6);       
		add_action('admin_menu', array($this, 'register_admin_menu'));
		add_filter('plugin_action_links_'.plugin_basename($mainPluginFile), array($this, 'plugin_list_settings_link') );
	}
	
	/**
	 *Update from version 1 of the plugin - remove the old database options, add the new ones, and generate the default CSS file
	 */
	public function update(){
		//create a css file based on the current settings
		$this->saveCSS($this->generateCSSpreset($this->_options['size'], $this->_options['png'], $this->_options['title-pos']));
		$this->_options['opts_updated'] = time();
	
		//update the options to remove things no longer needed now we're using user modifiable CSS
		unset($this->_options['css'], 
			$this->_options['png'],
			$this->_options['size'],
			$this->_options['title-pos']);
		//add in the two new link options
		$this->_options['links']['delicious'] = false;
		$this->_options['links']['reddit'] = false;
		//and the manual location option
		$this->_options['manual_location'] = false;
		update_option( 'xoogu-simple-social-sharing', $this->_options);
	}
	
	/**
	 *This function isn't used, but poedit should pick up the strings to be translated
	 */
	private function getTextSocialNetworkNames(){
		__('Facebook', 'xoogu-simple-social-sharing');
		__('Twitter', 'xoogu-simple-social-sharing');
		__('Gplus', 'xoogu-simple-social-sharing');
		__('Linkedin', 'xoogu-simple-social-sharing');
		__('Pinterest', 'xoogu-simple-social-sharing');
		__('Delicious', 'xoogu-simple-social-sharing');
		__('Reddit', 'xoogu-simple-social-sharing');
	}
	
	/**
	 *Saves the default settings to the DB if no settings exist
	 *and creates a CSS file in the site's uploads directory with the default CSS
	 *See http://www.wphub.com/tutorials/plugins-compatible-wordpress-multisite/
	 */
	public function activate( $network_wide=false ){
		ob_start();
		if(!current_user_can('activate_plugins')){
			if ( defined( 'WP_DEBUG_LOG' ) && WP_DEBUG_LOG ) {
				error_log( __('Can\'t activate xoogu simple social sharing plugin as user lacks the \'activate_plugins\' permission', 'xoogu-simple-social-sharing') );
			}
			wp_die( __( 'You do not have sufficient permissions to manage plugins for this site.', 'xoogu-simple-social-sharing' ) );
		}
		
		$defaultOpts = array(
			//apparently checking an array key isset is much faster than checking a value is in_array: http://www.phpreferencebook.com/tips/isset-in_array-mysql/
			//We don't actually use isset for checking though, as it makes saving options easier if we just use bool values for on/off items
			'links'=>array(
				'facebook'=>true, 'twitter'=>true, 'gplus'=>true, 'linkedin'=>true, 'pinterest'=>true, 'delicious' => false, 'reddit' => false
			),
			'js' => true,//'jquery',
			'pos-before' => false,
			'pos-after' => true,
			'title' => 'Share:',
			'single' => 'single',
			'manual_location' => false,
			'opts_updated' => time()
		);
		if ( !is_multisite() || !$network_wide ){
			$this->activate_site($defaultOpts);
		} 
		// For Multisite
		else 
		{
			global $wpdb;
			$blog_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );
			$original_blog_id = get_current_blog_id();
			foreach ( $blog_ids as $blog_id ) 
			{
				switch_to_blog( $blog_id );
				$this->_options = get_option('xoogu-simple-social-sharing');
				$this->activate_site($defaultOpts);
			}
			switch_to_blog( $original_blog_id );
			$this->_options = get_option('xoogu-simple-social-sharing');
		}
		file_put_contents('/home/djeyewater/Desktop/debug.txt', ob_get_clean());
	}
	
	/**
	 *Saves the default settings to the database and creates a CSS file in the site's uploads directory with the default CSS
	 *@param array $defaultOpts The default options to save
	 */
	public function activate_site($defaultOpts){
		//has the plugin been activated previously
		if(!empty($this->_options)){
			//if we're updating from an older version
			if(isset($this->_options['png'])){ //this setting is in v1 but not v2, so if it exists then must be v1
				$this->update();
			}
			return;
		}
		
		//we'll try and detect jQuery usage in a rather hacky way
		//Since we're in the admin area, jquery will be loaded whether the actual site uses it or not
		//So we need to make an HTTP request with no cookies (so admin scripts aren't loaded) to a standard page
		//Easiest way to do this is to make an HTTP request from the server side, so that's what we'll do
		//Not foolproof as all we're doing is checking for presence of 'jquery' in script src
		//if a script is named no-jquery.js then we'll think we do have jquery when we don't
		//or if a script contains jquery (like a minified and concatenated script) but not in the filename, then we'll think we don't have jquery when we do
		
		//set a custom error handler to avoid warnings from DOMDocument printing to page (causes 'Plugin generated x characters of output' message in WP when activating the plugin)
		set_error_handler(function($errno, $errstr, $errfile, $errline) { if ( defined( 'WP_DEBUG_LOG' ) && WP_DEBUG_LOG ) {error_log("$errstr $errfile:$errline");} });
		try{
			//get a post
			$recent_posts = wp_get_recent_posts(array('numberposts' => 1, 'post_status' => 'publish'), ARRAY_A);
			//make an HTTP request to fetch the page
			$response=wp_remote_get(get_permalink($recent_posts[0]['ID']));
			//var_dump($response);
			//load the result into a DOM Document
			$dom = new DOMDocument;
			$dom->loadHTML($response['body']);
			//Find the script tags
			$scripts = $dom->getElementsByTagName('script');
			//check to see if jquery is listed
			foreach($scripts as $script){
				if(stripos($script->getAttribute('src'), 'jquery')){
					//we (probably) found jquery, so set it as the default for js type
					$defaultOpts['js']='jquery';
					break;
				}
			}
		}
		catch(Exception $e){
			//if we don't have DOMDocument available or it couldn't parse the page probably
			//no need to do anything, we'll keep js value as the default
		}
		restore_error_handler();
		//save the default settings to the db
		add_option( 'xoogu-simple-social-sharing', $defaultOpts);
		//create a css file based on the default settings
		$this->saveCSS($this->generateCSSpreset());
	}
	
	
	/**
	 *If the plugin is network activated, then activate it for any new blog when it is added (hook this function into wpmu_new_blog)
	 *@param int $blog_id The id of the newly added blog
	 *@param int $user_id id of blog administrator
	 *@param string $domain Domain name for blog
	 *@param string $path Path to the blog
	 *@param int $site_id Site id, if running multiple networks
	 *@param array $meta Array of blog meta (options)
	 */
	function new_blog($blog_id, $user_id, $domain, $path, $site_id, $meta ) {
		if (is_plugin_active_for_network(plugin_basename(dirname(__FILE__).'/xoogu-simple-social-sharing.php'))) {
			$original_blog_id = get_current_blog_id();
			switch_to_blog($blog_id);
			$this->_options=array(); //options currently loaded are for current site so need to get rid of them
			$this->activate(false);
			switch_to_blog($original_blog_id);
		}
	}
	

	public function deactivate(){
	
	}
	
	/**
	 *Removes the database and CSS files when the plugin is uninstalled
	 */
	public static function uninstall(){
		if ( !is_multisite() ){
			self::uninstall_site();
		} 
		// For Multisite
		else{
			global $wpdb;
			$blog_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );
			$original_blog_id = get_current_blog_id();
			foreach ( $blog_ids as $blog_id ) {
				switch_to_blog( $blog_id );
				self::uninstall_site();
			}
			switch_to_blog( $original_blog_id );
		}
	}
	
	/**
	 *Removes the database and CSS files when the plugin is uninstalled
	 */
	public static function uninstall_site(){
		//delete the css file
		$upload_dir = wp_upload_dir( date('Y') );
		unlink($upload_dir['basedir'].'/xoogu-simple-social-sharing.css');
		unlink($upload_dir['basedir'].'/xoogu-simple-social-sharing-not-min.css');
		//remove the database entry
		delete_option( 'xoogu-simple-social-sharing' );  
	}
	
	/**
	 *Adds the plugin settings page to the settings menu
	 */
	public function register_admin_menu(){
		//Add a new menu item under Settings
		$hook_suffix = add_options_page(
			/* translators: Page title - The text to be displayed in the title tags of the page when the menu is selected */
			__('xoogu Simple Social Sharing', 'xoogu-simple-social-sharing'),
			/* translators: Menu title - The text to be used for the menu */
			__('xoogu Simple Social Sharing', 'xoogu-simple-social-sharing'),
			'manage_options', 'xoogu-simple-social-sharing', array($this, 'display_page') );
		//Using registered $page handle to hook stylesheet loading
        add_action( 'load-' . $hook_suffix, array($this, 'admin_load') );
	}
	
	/**Adds a settings link in the plugin details on the plugin page (filter applied to plugin_action_links_$name)
	 *@param array $links Links to print at the bottom of the plugin details
	 *@return array Links to print at the bottom of the plugin details
	 */
	public function plugin_list_settings_link($links) {
		array_unshift($links, '<a href="options-general.php?page=xoogu-simple-social-sharing">'.__('Settings', 'xoogu-simple-social-sharing').'</a>'); 
		return $links; 
	}
	
	/**
	 *Displays the settings page
	 */
	public function display_page(){
		global $post;
		
		//to generate the preview we need to set up the loop
		//http://codex.wordpress.org/Function_Reference/query_posts states "don't use query_posts() ever"
		/*query_posts( array( 'posts_per_page' => 1));
		the_post();
		$preview=$this->add_links('<p>The content</p>');
		wp_reset_query();*/
		//so we use WP_Query instead - http://codex.wordpress.org/Class_Reference/WP_Query
		$the_query = new WP_Query( array( 'posts_per_page' => 1) );
		$the_query->the_post();
		//We need to set all links to show for the preview to work
		$links = $this->_options['links'];
		foreach($this->_options['links'] as &$val){
			$val=1;
		}
		//create the preview
		$preview=$this->add_links('<p>'.__('This would be the content of your post', 'xoogu-simple-social-sharing').'</p>');
		echo '<div id="preview-container"><h2>'; _e('Preview', 'xoogu-simple-social-sharing'); echo '</h2><div>'; _e('You can see here how the PNG quality choice, choice of links, position, and size will affect the display. Note that the way it appears on the live website may differ due to your theme\'s CSS.', 'xoogu-simple-social-sharing'); echo '</div>
		<div id="xsss-preview" data-page-url="'.get_permalink().'" >'.$preview.'</div>
		</div>';
		wp_reset_postdata();
		//reset the links to the saved options
		$this->_options['links'] = $links;
		
		
		?>
		<hr />
		<form action="" method="post" enctype="multipart/form-data" autocomplete="off">
			<fieldset>
				<legend><?php _e('Settings', 'xoogu-simple-social-sharing');?></legend>
				<dl>
					<dt><?php _e('Show buttons for:', 'xoogu-simple-social-sharing');?></dt>
					<dd>
					<ul>
						<?php foreach($this->_options['links'] as $name => $val){
							echo'<li><input type="checkbox" id="links-'.$name.'" name="options[links]['.$name.']" '.($val ? 'checked="checked" ' : '').' /><label for="links-'.$name.'">'.__(ucfirst($name), 'xoogu-simple-social-sharing').'</label></li>';
						}?>
					</ul>
					</dd>
					<dt><?php _e('Use javascript to make the links open in a new window (recommended):', 'xoogu-simple-social-sharing');?></dt>
					<dd>
						<div class="desc"><?php _e("If you already have a plugin that includes jquery, or your theme uses jquery, it is recommended to use jquery here, as it is a smaller size. If your site doesn't already use jquery, then it is recommended to just use the standard option, which will avoid the overhead of loading the jquery library. (Please note this setting is not reflected in the preview above).", 'xoogu-simple-social-sharing');?></div>
						<ul>
							<li><input type="radio" value="" name="options[js]" id="js-0" <?php if(!$this->_options['js']){echo 'checked="checked" ';}?> /><label for="js-0"><?php _e('No', 'xoogu-simple-social-sharing');?></label></li>
							<li><input type="radio" value="1" name="options[js]" id="js-1" <?php if($this->_options['js']===true){echo 'checked="checked" ';}?> /><label for="js-1"><?php _e('Yes', 'xoogu-simple-social-sharing');?></label></li>
							<li><input type="radio" value="jquery" name="options[js]" id="js-jquery" <?php if($this->_options['js']==='jquery'){echo 'checked="checked" ';}?> /><label for="js-jquery"><?php _e('Yes, use jquery', 'xoogu-simple-social-sharing');?></label></li>
						</ul>
					</dd>
					<dt><?php _e('Position of buttons:', 'xoogu-simple-social-sharing');?></dt>
					<dd>
						<ul>
							<li><input type="checkbox" name="options[pos-before]" id="pos-before" <?php if($this->_options['pos-before']){echo 'checked="checked" ';}?> /><label for="pos-before"><?php _e('Before content', 'xoogu-simple-social-sharing');?></label></li>
							<li><input type="checkbox" name="options[pos-after]" id="pos-after" <?php if($this->_options['pos-after']){echo 'checked="checked" ';}?> /><label for="pos-after"><?php _e('After content', 'xoogu-simple-social-sharing');?></label></li>
							<li><input type="checkbox" name="options[manual_location]" id="manual_location" <?php if($this->_options['manual_location']){echo 'checked="checked" ';}?> /><label for="manual_location"><?php _e('Manual location specified in theme template', 'xoogu-simple-social-sharing');?></label></li>
						</ul>
					</dd>
					<dt><?php _e('Title of share:', 'xoogu-simple-social-sharing');?></dt>
					<dd><input type="text" name="options[title]" id="title" value="<?php echo $this->_options['title']; ?>" /><label for="title"><?php _e('Title', 'xoogu-simple-social-sharing');?></label></dd>
					<dt><?php _e('Show on:', 'xoogu-simple-social-sharing');?></dt>
					<dd>
						<ul>
							<li><input type="radio" name="options[single]" id="single-single" value="single" <?php if($this->_options['single']=='single'){echo 'checked="checked" ';} ?>/><label for="single-single"><?php _e('Show on single posts only', 'xoogu-simple-social-sharing');?></label></li>
							<li><input type="radio" name="options[single]" id="single-singular" value="singular" <?php if($this->_options['single']=='singular'){echo 'checked="checked" ';} ?>/><label for="single-singular"><?php _e('Show on single posts, pages, and attachment pages only', 'xoogu-simple-social-sharing');?></label></li>
							<li><input type="radio" name="options[single]" id="single-no" value="" <?php if(!$this->_options['single']){echo 'checked="checked" ';} ?>/><label for="single-no"><?php _e('Show on all posts, pages, and attachment pages', 'xoogu-simple-social-sharing');?></label></li>
						</ul>
					</dd>
					<dt><?php _e('CSS for displaying the buttons:', 'xoogu-simple-social-sharing');?></dt>
					<dd>
						<div class="desc"><?php _e('Choose from a preset and edit as you like', 'xoogu-simple-social-sharing');?></div>
						<div>
							<select id="css-preset" name="css-preset">
								<option value=""><?php _e('Choose a preset', 'xoogu-simple-social-sharing');?></option>
								<option value="64 32 0"><?php printf(__('%1$s %2$s title above', 'xoogu-simple-social-sharing'), '64px', 'PNG32');?></option>
								<option value="64 32 1"><?php printf(__('%1$s %2$s title left', 'xoogu-simple-social-sharing'), '64px', 'PNG32');?></option>
								<option value="64 8 0"><?php printf(__('%1$s %2$s title above', 'xoogu-simple-social-sharing'), '64px', 'PNG8');?></option>
								<option value="64 8 1"><?php printf(__('%1$s %2$s title left', 'xoogu-simple-social-sharing'), '64px', 'PNG8');?></option>
								<option value="32 32 0"><?php printf(__('%1$s %2$s title above', 'xoogu-simple-social-sharing'), '32px', 'PNG32');?></option>
								<option value="32 32 1"><?php printf(__('%1$s %2$s title left', 'xoogu-simple-social-sharing'), '32px', 'PNG32');?></option>
								<option value="32 8 0"><?php printf(__('%1$s %2$s title above', 'xoogu-simple-social-sharing'), '32px', 'PNG8');?></option>
								<option value="32 8 1"><?php printf(__('%1$s %2$s title left', 'xoogu-simple-social-sharing'), '32px', 'PNG8');?></option>
								<option value="16 32 0"><?php printf(__('%1$s %2$s title above', 'xoogu-simple-social-sharing'), '16px', 'PNG32');?></option>
								<option value="16 32 1"><?php printf(__('%1$s %2$s title left', 'xoogu-simple-social-sharing'), '16px', 'PNG32');?></option>
								<option value="16 8 0"><?php printf(__('%1$s %2$s title above', 'xoogu-simple-social-sharing'), '16px', 'PNG8');?></option>
								<option value="16 8 1"><?php printf(__('%1$s %2$s title left', 'xoogu-simple-social-sharing'), '16px', 'PNG8');?></option>
							</select>
							<label for="css-preset"><?php _e('Choose a preset', 'xoogu-simple-social-sharing');?></label>
						</div>
						<div>
							<p><label for="css"><?php _e('CSS:', 'xoogu-simple-social-sharing');?></label></p>
							<textarea id="css" name="css"><?php $upload_dir = wp_upload_dir( date('Y') ); echo file_get_contents($upload_dir['basedir'].'/xoogu-simple-social-sharing-not-min.css');?></textarea>
						</div>
						<input type="hidden" id="pluginURL" value="<?php echo plugins_url( '', __FILE__ ); ?>" />
					</dd>
				</dl>
				<?php submit_button();?>
			</fieldset>
		</form><?php
	}
	
	/**
	 *Load CSS, JS, process POST for admin page
	 */
	public function admin_load(){
		add_action('admin_enqueue_scripts', array($this, 'loadjscss'));
		add_action('admin_enqueue_scripts', array($this, 'loadAdminjscss'));
		if(!empty($_POST)){
			$this->_savePOST($_POST);
		}
	}
	
	/**
	 *Load the admin js and css
	 */
	public function loadAdminjscss(){
		wp_enqueue_style( 'wp-color-picker' );
		wp_enqueue_style( 'xoogu-simple-social-sharing-admin', plugins_url( 'admin.css' , __FILE__ ), null, 2 );
		wp_enqueue_script( 'xoogu-simple-social-sharing-admin', plugins_url( 'admin.js' , __FILE__ ), array('jquery','wp-color-picker'), 2, true );
		wp_localize_script( 'xoogu-simple-social-sharing-admin', 'xsssL10n', array(
			'previewBGcolor' => __('Select background colour for preview:', 'xoogu-simple-social-sharing'),
			'ie7comment' => __('IE7 & below', 'xoogu-simple-social-sharing')
		) );
	}
	
	/**
	 *Process the POST and save the settings to the DB. Note that if an option
	 *is missing from the POST, that option's value will be saved as false.
	 *@param array $POST The POSTed options and css to save ($_POST)
	 */
	protected function _savePOST($POST){
		
		//save the db options
		$postOpts = $POST['options'];
		$opts = &$this->_options;
		foreach($opts['links'] as $name =>&$val){
			$val=empty($postOpts['links'][$name]) ? false : true;
		}
		unset($val);
		//"0" will evaluate as being empty
		$opts['js'] = empty($postOpts['js']) ? false : ($postOpts['js']=='jquery' ? 'jquery' : true);
		$opts['pos-before'] = empty($postOpts['pos-before']) ? false : true;
		$opts['pos-after'] = empty($postOpts['pos-after']) ? false : true;
		$opts['title'] = $postOpts['title'];
		$opts['single'] = empty($postOpts['single']) ? false : $postOpts['single'];
		$opts['manual_location'] = empty($postOpts['manual_location']) ? false : true;
		$opts['opts_updated'] = time();
		update_option('xoogu-simple-social-sharing', $opts);
		
		//save the css (WP adds slashes to POST data to make things more complicated, so now we need to remove them)
		$this->saveCSS(stripslashes($POST['css']));
	}
	
	
	/**
     *Generate CSS for the plugin to use
     *@param int $size The size in pixels of the icons (16, 32, or 64)
     *@param int $bitDepth Use PNG8 or PNG32 image
     *@param bool $titlePos Whether the title should be positioned above (false) or to the left (true) of the icons
     */
	public function generateCSSpreset($size=32, $bitDepth=32, $titlePos=0){
		$css='
#xsss-buttons-before a, #xsss-buttons-after a{
	width: '.$size.'px;
	height: '.$size.'px;
	background-image: url("'.plugins_url( '/imgs/social-icons-PNG'.$bitDepth.'-'.$size.'px.png', __FILE__ ).'");
	display: inline-block;
	text-indent: 100%;
	white-space: nowrap;
	overflow: hidden;
	vertical-align: middle;
	/*'.__('IE7 & below', 'xoogu-simple-social-sharing').'*/
	*text-indent: 0;
	font-size:0;
	line-height: 0;
}
#xsss-buttons-before a:hover, #xsss-buttons-after a:hover{
	-moz-filter: brightness(1.1);
	-webkit-filter: brightness(1.1);
	-ms-filter: brightness(1.1);
	filter: brightness(1.1);
}
.xsss-fb{
	background-position: -'.($size*2).'px 0;
}
.xsss-t{
	background-position: '.($size*4).'px 0;
}
.xsss-g{
	background-position: '.($size*3).'px 0;
}
.xsss-l{
	background-position: -'.$size.'px 0;
}
.xsss-r{
	background-position: '.($size*2).'px 0;
}
.xsss-d{
	background-position: '.$size.'px 0;
}';
		if($titlePos){
			$css .= '
h4.xsss-title{
	display: inline-block;
	margin: 0 10px 0 0;
	/*'.__('IE7 & below', 'xoogu-simple-social-sharing').'*/
	*display: inline;
}';
		}else{
			$css .= '
h4.xsss-title{
	margin: 1em 0 0.5em 0;
}';
		}
		return $css;
	}
	
	
	/**
	 *Saves minimised and unminised versions of the provided CSS in the site's uploads folder
	 *@param string $css The CSS to save
	 */
	public function saveCSS($css){
		//save the unminified version for editing
		$upload_dir = wp_upload_dir( date('Y') );
		file_put_contents($upload_dir['basedir'].'/xoogu-simple-social-sharing-not-min.css', $css);
		
		//minify
		//remove comment blocks
		$css = preg_replace('/(?:\/\*(.|[\r\n])*?\*\/)/m', ' ', $css);
		//Match any of :{}; preceded or proceeded by whitespace(s) and strip the whitespace
		$css = preg_replace('/\s*([:{};])\s*/m', '$1', $css);
		//strip final semicolons
		$css=str_replace(';}', '}', $css);
		//Remove all whitespace that is more than 1 char in length
		$css = preg_replace('/\s{2,}/m', ' ', $css);
		
		//save the minified version
		file_put_contents($upload_dir['basedir'].'/xoogu-simple-social-sharing.css', $css);
	}

}
